/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.core.descriptor;

import java.io.File;
import java.util.logging.Level;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.core.Environment;
import org.apache.muse.core.routing.ResourceRouter;
import org.apache.muse.util.ReflectUtils;
import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class SimpleRouterDescriptor implements RouterDescriptor
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = 
        MessagesFactory.get(SimpleRouterDescriptor.class);
        
    private RouterDefinition _definition = null;
    
    protected InitParamDescriptor createInitParamDescriptor()
    {
        return new SimpleInitParamDescriptor();
    }
    
    protected File createLogFile(Element logging, Environment env)
    {
        QName qname = DescriptorConstants.LOG_FILE_QNAME;
        String location = XmlUtils.getElementText(logging, qname);
        return new File(env.getRealDirectory(), location);
    }
    
    protected LoggingConfig createLogging(Element xml, Environment env)
    {
        QName qname = DescriptorConstants.LOGGING_QNAME;
        Element loggingXML = XmlUtils.getElement(xml, qname);

        LoggingConfig logging = new LoggingConfig();
        
        File file = createLogFile(loggingXML, env);
        logging.setLogFile(file);
        
        Level level = createLogLevel(loggingXML);        
        logging.setLogLevel(level);
        
        return logging;
    }
    
    protected Level createLogLevel(Element logging)
    {
        //
        // get the logging level - this will be default for all resource 
        // instances created by the application
        //
        QName qname = DescriptorConstants.LOG_LEVEL_QNAME;
        String levelString = XmlUtils.getElementText(logging, qname);
        
        if (levelString == null)
            return Level.INFO;
        
        return Level.parse(levelString.trim());
    }
    
    protected PersistenceDefinition createPersistence(Element xml, Environment env)
        throws SoapFault
    {
        Element persistenceXML = XmlUtils.getElement(xml, DescriptorConstants.PERSISTENCE_QNAME);
        
        if (persistenceXML == null)
            return null;
        
        PersistenceDescriptor desc = createPersistenceDescriptor();
        desc.load(persistenceXML, env);
        
        return desc.getPersistenceDefinition();
    }
    
    protected PersistenceDescriptor createPersistenceDescriptor()
    {
        return new RouterPersistenceDescriptor();
    }
    
    protected Class createRouterClass(Element xml, Environment env)
    {
        String className = XmlUtils.getElementText(xml, DescriptorConstants.JAVA_ROUTER_QNAME);
        
        ClassLoader loader = env.getClassLoader();
        Class theClass = ReflectUtils.getClass(className, loader);
        
        //
        // make sure this class is an instance of ResourceRouter
        //
        if (!ResourceRouter.class.isAssignableFrom(theClass))
        {
            Object[] filler = { className, ResourceRouter.class.getName() };
            String message = _MESSAGES.get("IncorrectRouterRoot", filler);
            throw new RuntimeException(message);
        }
        
        return theClass;
    }
    
    public RouterDefinition getRouterDefinition()
    {
        return _definition;
    }
    
    public void load(Element xml, Environment environment)
        throws SoapFault
    {
        Class routerClass = createRouterClass(xml, environment);
        LoggingConfig logging = createLogging(xml, environment);
        PersistenceDefinition persistence = createPersistence(xml, environment);
        
        _definition = new RouterDefinition();
        _definition.setEnvironment(environment);
        _definition.setRouterClass(routerClass);
        _definition.setLoggingConfig(logging);
        _definition.setPersistenceDefinition(persistence);
    }
}
