/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.core.routing;

import java.lang.reflect.Method;

import javax.xml.namespace.QName;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;

/**
 *
 * AbstractMessageHandler is a convenience class that implements all of the 
 * accessor methods of {@linkplain MessageHandler MessageHandler}, leaving 
 * only the serialization methods (fromXML(Element) and toXML(Object)).
 *
 * @author Dan Jemiolo (danj)
 *
 */

public abstract class AbstractMessageHandler implements MessageHandler
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = 
        MessagesFactory.get(AbstractMessageHandler.class);
    
    /**
     * 
     * An empty array that can be shared to represent requests with zero 
     * parameters.
     * 
     */
    public static final Object[] EMPTY_REQUEST = new Object[0];
    
    //
    // The WS-A Action URI that maps to the operation
    //
    private String _actionURI = null;
    
    //
    // The actual Java method that will be called
    //
    private Method _method = null;
    
    //
    // The name of the request message (in the SOAP Body)
    //
    private QName _requestQName = null;
    
    //
    // The name of the response message (in the SOAP Body)
    //
    private QName _responseQName = null;
    
    /**
     * 
     * @param actionURI
     *        The WS-A Action URI this handler is used for.
     *        
     * @param requestQName
     *        The QName of the element that wraps parameters for incoming messages.
     *
     */
    protected AbstractMessageHandler(String actionURI, QName requestQName)
    {
        if (actionURI == null)
            throw new NullPointerException(_MESSAGES.get("NullAction"));

        _actionURI = actionURI;
        _requestQName = requestQName;
        
        if (_requestQName != null)
            _responseQName = createResponseName();
    }
    
    /**
     * 
     * Creates the QName that will be used to package up all objects that 
     * are serialized to XML with toXML(Object). The default behavior is 
     * to take the local part of the request QName and add the "Response" 
     * suffix to it. Thus, CreatePurchase becomes CreatePurchaseResponse. 
     * Users can override this method to change the response name.
     *
     * @return A QName that is just like the request QName, but with the 
     *         "Response" suffix added to the end of the local part.
     * 
     * @see #getRequestName()
     * @see #getResponseName()
     *
     */
    private QName createResponseName()
    {
        QName request = getRequestName();
        String uri = request.getNamespaceURI();
        String name = request.getLocalPart();
        String prefix = request.getPrefix();        
        return new QName(uri, name + "Response", prefix);
    }
    
    public String getAction()
    {
        return _actionURI;
    }
    
    public Method getMethod()
    {
        return _method;
    }
    
    public final QName getRequestName()
    {
        return _requestQName;
    }
    
    public QName getResponseName()
    {
        return _responseQName;
    }
    
    public void setMethod(Method method)
    {
        if (method == null)
            throw new NullPointerException(_MESSAGES.get("NullMethod"));
        
        _method = method;
    }
}
