/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.core.routing;

import java.lang.reflect.Method;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 *
 * MessageHandler defines the methods needed to serialize and deserialize the 
 * parameters and responses for a web service operation. It also references a 
 * Java method that is used to invoke the operation.
 * <br><br>
 * This interface is used for <b>receiving</b> incoming messages on the 
 * server-side, not the client-side. <b>Sending</b> web service requests is 
 * handled by {@linkplain org.apache.muse.core.proxy.ProxyHandler ProxyHandler}.
 * 
 * @author Dan Jemiolo (danj)
 *
 */

public interface MessageHandler
{
    /**
     * 
     * Deserializes the given DOM Element into a set of POJOs that can be 
     * used when invoking a Java method. Implementations should use Muse's 
     * registered Serializers in order to deserialize the XML into objects of 
     * the proper types.
     * 
     * @param xml
     *        The contents of a SOAP Body, where the root element contains 
     *        all of the method parameters. For simple parameters, this 
     *        element may contain the actual parameter data.
     * 
     * @return An array with the method parameters in the correct order for 
     *         invocation. This array can be passed to the invoke() method in 
     *         java.lang.reflect.Method. If the method takes no parameters, 
     *         an empty array is returned.
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If the number of parameters does not match the number of 
     *         child Elements.</li>
     *         <li>If no serialization mechanism exists for the object's type.</li>
     *         <li>If any child elements could not be deserialized into an object.</li>
     *         </ul>
     * 
     * @see org.apache.muse.core.serializer.Serializer#fromXML(Element)       
     *
     */
    Object[] fromXML(Element xml) 
        throws SoapFault;
    
    /**
     * 
     * @return The WS-A Action associated with the handler's operation.
     *
     */
    String getAction();
    
    /**
     * 
     * @return The Java method that should be used to execute the operation.
     *
     */
    Method getMethod();
    
    /**
     *
     * @return The QName of the SOAP request element that deserializable 
     *         DOM Elements will be wrapped in.
     *
     */
    QName getRequestName();
    
    /**
     *
     * @return The QName of the SOAP response element that deserializable 
     *         DOM Elements will be wrapped in.
     *
     */
    QName getResponseName();
    
    /**
     * 
     * @param method
     *        The Java method that should be used to execute the operation.
     *
     */
    void setMethod(Method method);
    
    /**
     * 
     * Serializes the given object into a DOM Element. Implementations should 
     * use Muse's registered Serializers in order to serialize the objects 
     * into the proper formats.
     *
     * @param result
     *        The response value from the operation, which must be serialized 
     *        into XML.
     * 
     * @return The XML equivalent of the given object, wrapped in an element 
     *         whose QName is the value of getResponseName().
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If no serialization mechanism exists for the object's type.</li>
     *         <li>If the object could not be serialized into XML.</li>
     *         </ul>
     * 
     * @see #getResponseName()
     * @see org.apache.muse.core.serializer.Serializer#toXML(Object, QName)
     *
     */
    Element toXML(Object result) 
        throws SoapFault;
}
