/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.core.routing;

import java.util.Collection;
import java.util.logging.Logger;

import org.w3c.dom.Element;

import org.apache.muse.core.Environment;
import org.apache.muse.core.Initialization;
import org.apache.muse.core.Shutdown;

/**
 *
 * ResourceRouter is the gateway to the generic resource layer that sits on 
 * top of the isolation layer. It is <em>generic</em> because it has no 
 * platform-specific dependencies -  it uses raw SOAP and WS-Addressing data 
 * to match requests with the resources being targeted by the client. The 
 * isolation layer simply extracts this data in the platform-specific way and 
 * then delegates control to the ResourceRouter, which further delegates to 
 * an individual {@linkplain org.apache.muse.core.Resource Resource object}. 
 * There should only be one ResourceRouter per application.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public interface ResourceRouter extends Initialization, Shutdown
{
    /**
     * 
     * @return The router's access point for things such as the 
     *         WS-Addressing context, file system resources, and more.
     *         
     */
    Environment getEnvironment();
    
    /**
     * 
     * @return The JDK logger that writes to the Muse log file.
     *
     */
    Logger getLog();
    
    /**
     * 
     * @return The persistence mechanism for the router, or null if none 
     *         was defined.
     *
     */
    RouterPersistence getPersistence();
    
    /**
     * 
     * @return The collection of ResourceDefinition objects representing 
     *         all of the resource types that the router controls.
     *
     */
    Collection getResourceDefinitions();
    
    /**
     * 
     * This is a generic method invocation mechanism that can be used to 
     * delegate request handling to a specific Resource. This method should be 
     * used by the isolation layer in order to process external requests.
     * 
     * @param soapBody
     *        The parameters for the method, still in their SOAP Body form.
     * 
     * @return The return value of the method, in SOAP Body form. If there was 
     *         an error while processing the request, the value will be a 
     *         SOAP v1.2 fault, in XML form.
     *  
     * @see org.apache.muse.core.Resource#invoke(Element)
     *
     */
    Element invoke(Element soapBody);
    
    void setEnvironment(Environment environment);
    
    void setLog(Logger log);
    
    void setPersistence(RouterPersistence persistence);
    
    void setResourceDefinitions(Collection definitions);
}
