/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.tools.generator.util;

import java.io.File;

import org.apache.muse.core.descriptor.DescriptorConstants;
import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.wsdl.WsdlUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Helps the projectizes create and analyze deployment (muse.xml) descriptors.
 * This really comes into play when we are passed a deployment descriptor for code generation
 * that has multiple resource types. This class figured out which WSDL goes with which resource type.
 * 
 * @author Andrew Eberbach (aeberbac)
 */
public class DeploymentDescriptorHelper {

	private Document _descriptorDocument;
	private Element _wsdlFileElement;
	private Element _resourceTypeElement;
	private Element _contextPathElement;	
	private Element _javaResourceClass;
	private static Messages _MESSAGES = MessagesFactory.get(DeploymentDescriptorHelper.class);

	/**
	 * Given a descriptor, wsdl and resource index as XML look through the 
	 * document, find the resourceIndex'th resource and attach to it.
	 * 
	 * @param descriptorDocument
	 * @param wsdlDocument
	 * @param resourceIndex
	 * 
	 * @throws Exception
	 */
	public DeploymentDescriptorHelper(Document descriptorDocument, Document wsdlDocument, int resourceIndex) throws Exception {
		_descriptorDocument = descriptorDocument;		
		_resourceTypeElement = findResourceTypeElement(_descriptorDocument, wsdlDocument, resourceIndex);
		_wsdlFileElement = XmlUtils.findFirstInSubTree(_resourceTypeElement, DescriptorConstants.WSDL_FILE_QNAME);
		_contextPathElement = XmlUtils.findFirstInSubTree(_resourceTypeElement, DescriptorConstants.CONTEXT_PATH_QNAME);
		_javaResourceClass = XmlUtils.findFirstInSubTree(_resourceTypeElement, DescriptorConstants.JAVA_RESOURCE_QNAME);
	}

	private Element findResourceTypeElement(Document descriptorDocument, Document wsdlDocument, int resourceIndex) throws Exception {
		Element portTypeElement = XmlUtils.findFirstInSubTree(wsdlDocument.getDocumentElement(), WsdlUtils.PORT_TYPE_QNAME );
		
		if(portTypeElement == null) {
			throw new Exception(_MESSAGES .get("NoPortType"));
		}
		
		String portTypeName = portTypeElement.getAttribute(WsdlUtils.NAME);
		
		if(portTypeName == null) {
			throw new Exception(_MESSAGES .get("NoPortTypeName"));
		}
		
		String portTypeURI = wsdlDocument.getDocumentElement().getAttribute(XmlUtils.TARGET_NS);
		
		if(portTypeURI == null) {
			throw new Exception(_MESSAGES .get("NoTargetNamespace"));
		}
		
		Element[] resourceTypes = XmlUtils.findInSubTree(_descriptorDocument.getDocumentElement(), DescriptorConstants.RESOURCE_TYPE_QNAME);
		
		if(resourceTypes.length < 1) {
			throw new Exception(_MESSAGES.get("NoResourceTypeElements"));
		}
		
		if(resourceIndex >= resourceTypes.length) {
			Object[] filler = { new Integer(resourceIndex), new Integer(resourceTypes.length) };
			throw new IndexOutOfBoundsException(_MESSAGES.get("InvalidResourceIndex", filler));
		}
		

		if(findPortTypeElement(resourceTypes[resourceIndex], portTypeName, portTypeURI) != null) {
			return resourceTypes[resourceIndex];
		}
		
		if(resourceTypes.length != 1) {
			Object[] filler = { portTypeName, portTypeURI };
			throw new Exception(_MESSAGES.get("NoPortTypeMultipleCapabilities",filler));
		}
		
		portTypeElement = XmlUtils.findFirstInSubTree(resourceTypes[0], DescriptorConstants.WSDL_PORT_TYPE_QNAME);
		
		if(portTypeElement == null) {
			throw new Exception(_MESSAGES .get("NoPortTypeElement"));
		}		
		
		String prefix = "pfx";
		XmlUtils.setElementText(portTypeElement, prefix  + ":" + portTypeName);
		XmlUtils.setNamespaceAttribute(portTypeElement,prefix , portTypeURI);
		
		return resourceTypes[0];
	}

	private Element findPortTypeElement(Element resourceTypeElement, String portTypeName, String portTypeURI) throws Exception {
		Element portTypeElement = XmlUtils.findFirstInSubTree(resourceTypeElement, DescriptorConstants.WSDL_PORT_TYPE_QNAME);
		
		if(portTypeElement == null) {
			Object[] filler = { XmlUtils.toString(resourceTypeElement) };
			throw new Exception(_MESSAGES.get("NoPortTypeElement", filler));
		}
		
		String qname = XmlUtils.extractText(portTypeElement);
		
		if(qname == null) {
			return null;
		}
		
		String namespace = XmlUtils.resolveNamespace(qname, portTypeElement);
		
		if(namespace == portTypeURI && namespace == null) {
			return portTypeElement;
		}
		
		if(portTypeURI.equals(namespace)) {
			return portTypeElement;
		}
		
		return null;
	}

	public String getCapabilityClass(String uri) {
		Element capabilityElement = findCapabilityElement(uri);
		
		if(capabilityElement == null) {
			return null;
		}
		
		Element javaResourceClassElement = XmlUtils.findFirstInSubTree(capabilityElement, DescriptorConstants.JAVA_CAPABILITY_QNAME);
		
		if(javaResourceClassElement != null) {
			return XmlUtils.extractText(javaResourceClassElement);				
		}		
		
		return null;
	}
	
	private Element findCapabilityElement(String uri) {
		Element[] capabilities = XmlUtils.findInSubTree(_resourceTypeElement, DescriptorConstants.CAPABILITY_QNAME);
		for(int i=0; i < capabilities.length; i++) {
			Element capabilityURIElement = XmlUtils.findFirstInSubTree(capabilities[i],DescriptorConstants.CAPABILITY_URI_QNAME);
			String capabilityURI = XmlUtils.extractText(capabilityURIElement);	
			if(uri.equals(capabilityURI)) {
				return capabilities[i];
			}
		}
		return null;
	}

	public void setWsdlFile(String wsdlFileName) {		
		XmlUtils.setElementText(_wsdlFileElement, wsdlFileName);
	}

	public void setContextPath(String contextPathName) {
		XmlUtils.setElementText(_contextPathElement, contextPathName);
	}

	public void setJavaResourceClass(String name) {		
		XmlUtils.setElementText(_javaResourceClass, name);
	}

	public void addCapability(Capability capability) {
		Element capabilityElement = findCapabilityElement(capability.getURI());
		
		if(capabilityElement == null) {
			capabilityElement = XmlUtils.createElement(_descriptorDocument, DescriptorConstants.CAPABILITY_QNAME);
			Element capabilityUriElement = XmlUtils.createElement(_descriptorDocument, DescriptorConstants.CAPABILITY_URI_QNAME, capability.getURI() );
			
			capabilityElement.appendChild(capabilityUriElement);
			_resourceTypeElement.appendChild(capabilityElement);
		}
									
		Element javaResourceClassElement = XmlUtils.findFirstInSubTree(capabilityElement, DescriptorConstants.JAVA_CAPABILITY_QNAME);
		if(javaResourceClassElement == null) {
			javaResourceClassElement = XmlUtils.createElement(_descriptorDocument, DescriptorConstants.JAVA_CAPABILITY_QNAME);
			capabilityElement.appendChild(javaResourceClassElement);
		}
		
		XmlUtils.setElementText(javaResourceClassElement, capability.getImplementingClass());
	}

	public static File[] getWsdls(File deploymentDescriptorFile, Document deploymentDescriptorDocument) throws Exception {
		Element[] wsdlFileElements = XmlUtils.findInSubTree(deploymentDescriptorDocument.getDocumentElement(), DescriptorConstants.WSDL_FILE_QNAME);
		File[] files = new File[wsdlFileElements.length];
		
		for(int i=0; i < wsdlFileElements.length; i++) {
			String wsdlFileName = XmlUtils.extractText(wsdlFileElements[i]);
			if(wsdlFileName == null || wsdlFileName.length() == 0) {
				Object[] filler = { deploymentDescriptorFile.getAbsolutePath() };
				throw new Exception(_MESSAGES.get("EmptyWsdlFileElement", filler));							
			}
			
			files[i] = new File(deploymentDescriptorFile.getAbsoluteFile().getParentFile(), wsdlFileName);
			if(!files[i].exists()) {
				Object[] filler = { files[i].getAbsolutePath() };
				throw new Exception(_MESSAGES.get("RelativeWsdlFileNotFound", filler));
			}
		}
		
		return files;
	}
}
