/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.addressing.soap;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.xml.XmlSerializable;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.soap.SoapConstants;

/**
 * 
 * This class represents a the SOAP fault type defined in SOAP 1.2.
 * 
 * @author Dan Jemiolo (danj)
 *
 */

public class SoapFault extends Exception implements XmlSerializable
{
    private static final long serialVersionUID = -8612773172898054578L;
    
    private QName _code = SoapConstants.RECEIVER_QNAME;
    
    private QName _subCode = null;
    private String _reason = null;
    private String _nodeURI = null;
    private String _roleURI = null;
    
    private Element _detail = null;
    
    public SoapFault(Element xml)
    {
        Element codeXML = XmlUtils.getElement(xml, SoapConstants.CODE_QNAME);
        _code = XmlUtils.getQNameFromChild(codeXML, SoapConstants.VALUE_QNAME);
        
        Element subCodeXML = XmlUtils.getElement(codeXML, SoapConstants.SUBCODE_QNAME);
            
        if (subCodeXML != null)
            _subCode = XmlUtils.getQNameFromChild(subCodeXML, SoapConstants.VALUE_QNAME);
        
        Element reasonXML = XmlUtils.getElement(xml, SoapConstants.REASON_QNAME);
        _reason = XmlUtils.getElementText(reasonXML, SoapConstants.TEXT_QNAME);
        
        _nodeURI = XmlUtils.getElementText(xml, SoapConstants.NODE_QNAME);
        _roleURI = XmlUtils.getElementText(xml, SoapConstants.ROLE_QNAME);
        
        Element detailXML = XmlUtils.getElement(xml, SoapConstants.DETAIL_QNAME);
        
        if (detailXML != null)
            _detail = XmlUtils.getFirstElement(detailXML);
    }
    
    public SoapFault(String message)
    {
        _reason = message;
    }
    
    public SoapFault(String message, Throwable cause)
    {
        _reason = message;
        initCause(cause);        
    }
    
    public SoapFault(Throwable cause)
    {
        _reason = cause.getMessage();
        initCause(cause);
    }
    
    public final QName getCode()
    {
        return _code;
    }

    public final void setCode(QName code)
    {
        _code = code;
    }

    public final Element getDetail()
    {
        return _detail;
    }

    public final void setDetail(Element detail)
    {
        _detail = detail;
    }

    public final String getNodeURI()
    {
        return _nodeURI;
    }

    public final void setNodeURI(String nodeURI)
    {
        _nodeURI = nodeURI;
    }

    public final String getReason()
    {
        return _reason;
    }

    public final void setReason(String reason)
    {
        _reason = reason;
    }

    public final String getRoleURI()
    {
        return _roleURI;
    }
    
    public final void setRoleURI(String roleURI)
    {
        _roleURI = roleURI;
    }
    
    public final QName getSubCode()
    {
        return _subCode;
    }
    
    public final void setSubCode(QName subCode)
    {
        _subCode = subCode;
    }

    public String getMessage()
    {
        return getReason();
    }

    public Element toXML()
    {
        return toXML(XmlUtils.EMPTY_DOC);
    }

    public Element toXML(Document doc)
    {
        Element faultXML = XmlUtils.createElement(doc, SoapConstants.FAULT_QNAME);
        
        Element codeXML = XmlUtils.createElement(doc, SoapConstants.CODE_QNAME);
        faultXML.appendChild(codeXML);
        
        XmlUtils.setElement(codeXML, SoapConstants.VALUE_QNAME, getCode());
        
        QName subCode = getSubCode();
        
        if (subCode != null)
        {
            Element subCodeXML = XmlUtils.createElement(doc, SoapConstants.SUBCODE_QNAME);
            codeXML.appendChild(subCodeXML);
            XmlUtils.setElement(subCodeXML, SoapConstants.VALUE_QNAME, subCode);
        }
        
        Element reasonXML = XmlUtils.createElement(doc, SoapConstants.REASON_QNAME);
        faultXML.appendChild(reasonXML);
        XmlUtils.setElement(reasonXML, SoapConstants.TEXT_QNAME, getReason());
        
        String nodeURI = getNodeURI();
        
        if (nodeURI != null)
            XmlUtils.setElement(faultXML, SoapConstants.NODE_QNAME, nodeURI); 
        
        String roleURI = getRoleURI();
        
        if (roleURI != null)
            XmlUtils.setElement(faultXML, SoapConstants.ROLE_QNAME, roleURI); 
        
        Element detail = getDetail();
        
        if (detail != null)
            XmlUtils.setElement(faultXML, SoapConstants.DETAIL_QNAME, detail, false);
        
        return faultXML;
    }
}
