/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.dm.muws;

import javax.xml.namespace.QName;

import org.apache.muse.ws.resource.WsResource;
import org.apache.muse.ws.resource.basefaults.BaseFault;

/**
 * 
 * Relationships is an interface that represents the WSDM Relationships 
 * capability. It provides access to the capability's one resource property 
 * (muws2:Relationship) as well as methods for creating, destroying, and 
 * querying relationships.
 * <br><br>
 * The QueryRelationshipsByType operation is exposed here and can be exposed 
 * using the resource's WSDL. The other methods are for internal relationship 
 * management and should not be made public. If one desires to make the 
 * creation or manipulation of relationships possible through public APIs, he 
 * should do so as a side effect of another operation; that is, the generic 
 * methods for manipulation <b>all</b> relationships should remain private, 
 * while individual relationships may be altered using operations created 
 * specifically for that purpose.
 * 
 * @author Dan Jemiolo (danj)
 * 
 */

public interface Relationships extends ManageabilityCapability 
{
    QName[] PROPERTIES = new QName[]{
        MuwsConstants.RELATIONSHIP_QNAME
    };
    
    /**
     * 
     * Creates a new Relationship resource with the given values, adds it to 
     * the ResourceManager and the participant resources' WS-RP container, and 
     * returns it to the caller.
     * 
     * @param name
     *        The descriptive name of the new relationship.
     * 
     * @param type
     *        The XSD-defined type of the new relationship.
     * 
     * @param participants
     *        The participant resources for the relationship. The array 
     *        must have a length of at least 2.
     * 
     * @return The newly-created (and stored) Relationship resource.
     * 
     * @throws BaseFault
     *         <ul>
     *         <li>If there was an error during the initialization of the 
     *         Relationship resource instance.</li>
     *         <li>If there was an error inserting into the participant's 
     *         WS-RP container.</li>
     *         </ul>
     *
     */
    WsResource addRelationship(String name, RelationshipType type, Participant[] participants)
        throws BaseFault;
    
    /**
     * 
     * @return The resources that represent the relationships this resource 
     *         participates in. This is also the muws2:Relationship property. 
     *         The implementation of this capability must provide a means of 
     *         converting the WsResource objects into the proper XML format 
     *         for WSRP clients.
     * 
     * 
     */
    WsResource[] getRelationship()
        throws BaseFault;

    /**
     * 
     * This method is defined by MUWS Part 2 in order to optimize the 
     * query of a resource property that will often have a large number 
     * of instances (which would make XPath-like queries and catch-all Get 
     * requests a performance bottleneck). The functional equivalent of this 
     * call would be WS-RP QueryResourceProperties with an XPath query that 
     * tries to match the value of the relationship's muws2:Type field.
     * 
     * @param types
     *        The names of the types of relationships that should be 
     *        returned by the query processor.
     * 
     * @return An array of resources that can be used to manipulate the 
     *         relationships that match the query. The array may be empty.
     * 
     */
    WsResource[] queryRelationshipsByType(QName[] types)
        throws BaseFault;
    
    /**
     * 
     * Deletes the RelationshipResource from the owner's WS-RP container.
     *
     * @param relationship
     *
     */
    void removeRelationship(WsResource relationship)
        throws BaseFault;
}
