/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.dm.muws.impl;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.dm.muws.Match;
import org.apache.muse.ws.dm.muws.MuwsConstants;
import org.apache.muse.ws.resource.WsResource;

/**
 *
 * MatchFactory is a device for creating pbm:Match expressions from their 
 * XML definitions without having to deal with concrete pbm:Match types.
 * Because this factory has no state and the PBM types are finite, the 
 * class is exposed as a singleton.
 * 
 * @author Dan Jemiolo (danj)
 *
 */

public class MatchFactory
{
    //
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(MatchFactory.class);
    
    private static final MatchFactory _SINGLETON = new MatchFactory();
    
    /**
     * 
     * @return The singleton instance of this class.
     *
     */
    public static MatchFactory getInstance()
    {
        return _SINGLETON;
    }
    
    /**
     * 
     * The default constructor is made private to prevent instantiation. All 
     * users should use the singleton instance.
     *
     */
    private MatchFactory()
    {
        //
        // this only exists so we can make it private
        //
    }
    
    /**
     * 
     * @param resource
     *        The resource that is defining the CorrelatableProperties 
     *        capability and thus, this correlation expression.
     * 
     * @param xml
     *        The DOM Element representing the simple pbm:Match.
     * 
     * @return A Match, which may have one or more sub-Matches inside it.
     *
     */
    public Match createMatch(WsResource resource, Element xml)
    {
        if (resource == null)
            throw new NullPointerException(_MESSAGES.get("NullOwner"));
        
        if (xml == null)
            throw new NullPointerException(_MESSAGES.get("NullCPElement"));
        
        QName qname = XmlUtils.getElementQName(xml);
        
        //
        // no table - there are only three PBM types allowed
        //
        
        if (qname.equals(MuwsConstants.MATCH_QNAME))
            return new SimpleMatch(resource, xml);
        
        else if (qname.equals(MuwsConstants.MATCH_ALL_QNAME))
            return new AllMatch(resource, xml);
        
        else if (qname.equals(MuwsConstants.MATCH_ANY_QNAME))
            return new AnyMatch(resource, xml);
        
        String message = _MESSAGES.get("InvalidMatch", new Object[]{ qname });
        throw new IllegalArgumentException(message);
    }
}
