/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.notification;

import java.util.Collection;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.core.serializer.Serializer;
import org.apache.muse.util.xml.XmlSerializable;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.notification.faults.TopicExpressionDialectUnknownFault;
import org.apache.muse.ws.notification.topics.WstConstants;

/**
 * 
 * NotificationMessage represents the wsnt:NotificationMessageHolderType type 
 * defined by the WS-Notification 1.3 schema. It is used as part of the 
 * wsnt:Notify and wsnt:GetCurrentMessage operations.
 *
 * @author Dan Jemiolo (danj)
 *
 */
public interface NotificationMessage extends XmlSerializable
{
    /**
     * 
     * This method allows you to add message content in its XML form - the 
     * element will be added as the last child under the wsnt:Message part 
     * of the notification message.
     *
     * @param content
     *
     */
    void addMessageContent(Element content);

    /**
     * 
     * This method allows you to add message content that has not yet been 
     * serialized into XML. The content object must be of a type that has 
     * a Muse Serializer registered for it; these types include all of 
     * the base/built-in types plus those that had serializers registered 
     * for them in muse.xml.
     *
     * @param content
     * @param qname
     * 
     * @see Serializer
     *
     */
    void addMessageContent(QName qname, Object content) 
        throws SoapFault;
    
    /**
     * 
     * @param qname
     * 
     * @return The element under wsnt:Message that has the given name, or null 
     *         if no such element exists.
     *
     */
    Element getMessageContent(QName qname);
    
    /**
     * 
     * This is a convenience method that does the same task as 
     * getMessageContent(QName) but deserializes the message content into 
     * a POJO of the given type before returning it.
     *
     */
    Object getMessageContent(QName qname, Class type)
        throws SoapFault;
    
    /**
     * 
     * @return The names of all elements added under the wsnt:Message section.
     *
     */
    Collection getMessageContentNames();
    
    /**
     * 
     * @return The EPR of the resource that is sending the message. This may 
     *         be null.
     *
     */
    EndpointReference getProducerReference();
    
    /**
     * 
     * @return The EPR of the subscription that is causing the message to be 
     *         sent to the subscriber. This may be null.
     *
     */
    EndpointReference getSubscriptionReference();
    
    /**
     * 
     * @return The topic that this message was published under. This may be null.
     *
     */
    QName getTopic();

    /**
     *
     * @return The concrete expression dialect
     * 
     * @see WstConstants#CONCRETE_TOPIC_URI
     *
     */
    String getTopicDialect();

    void setProducerReference(EndpointReference producer);

    void setSubscriptionReference(EndpointReference subscription);

    void setTopic(QName topicPath);
    
    void setTopicDialect(String dialect) 
        throws TopicExpressionDialectUnknownFault;
}
