/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.notification.impl;

import javax.xml.namespace.QName;

import org.w3c.dom.Element;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.notification.Filter;
import org.apache.muse.ws.notification.WsnConstants;
import org.apache.muse.ws.notification.faults.*;
import org.apache.muse.ws.resource.WsResource;

/**
 * 
 * FilterFactory is a component for creating WS-Notification filters from XML 
 * fragments that are compliant with the WS-N v1.3 schema. It allows for the 
 * creation of topic, message, and properties filters. It is a stateless parser, 
 * so it is represented as a singleton.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class FilterFactory
{
    //
    // Used to look up all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(FilterFactory.class);
    
    private static final FilterFactory _SINGLETON = new FilterFactory();
    
    private FilterFactory()
    {
        //
        // only present to prevent extra instantiation
        //
    }
    
    /**
     * 
     * @return The singleton instance of the factory component.
     *
     */
    public static FilterFactory getInstance()
    {
        return _SINGLETON;
    }
    
    /**
     * 
     *
     * @param xml
     *        An XML fragment representing the WS-N filter, or null.
     *        
     * @param resource
     *        The resource that is producing messages and using the filter 
     *        to determine if a message should be sent to a subscriber.
     *        
     * @return A filter based on topics, message patterns, or property values. 
     *         If the first parameter was null, the method returns an instance 
     *         of PublishAllMessagesFilter.
     * 
     * @throws InvalidFilterFault
     * @throws TopicExpressionDialectUnknownFault
     * @throws InvalidTopicExpressionFault
     * @throws InvalidProducerPropertiesExpressionFault
     * @throws InvalidMessageContentExpressionFault
     *
     */
    public Filter newInstance(Element xml, WsResource resource)
        throws InvalidFilterFault, 
               TopicExpressionDialectUnknownFault, 
               InvalidTopicExpressionFault, 
               InvalidProducerPropertiesExpressionFault, 
               InvalidMessageContentExpressionFault
    {
        //
        // no filter? publish all messages
        //
        if (xml == null)
            return PublishAllMessagesFilter.getInstance();
        
        //
        // no table with multiple parsers - we only have three filter types, 
        // so just do all the parsing here
        //
        
        //
        // topic-based filtering?
        //
        Element topic = XmlUtils.getElement(xml, WsnConstants.TOPIC_EXPRESSION_QNAME);
        
        if (topic != null)
        {
            String dialect = topic.getAttribute(WsnConstants.DIALECT);
            QName topicName = XmlUtils.getQName(topic);
            return new TopicFilter(topicName, dialect);
        }
        
        //
        // resource properties-based filtering?
        //
        Element props = XmlUtils.getElement(xml, WsnConstants.PRODUCER_PROPERTIES_QNAME);
        
        if (props != null)
        {
            String dialect = props.getAttribute(WsnConstants.DIALECT);
            String expression = XmlUtils.extractText(props);
            return new ProducerPropertiesFilter(resource, expression, dialect);
        }
        
        //
        // message pattern (xpath)-based filtering?
        //
        Element content = XmlUtils.getElement(xml, WsnConstants.MESSAGE_CONTENT_QNAME);
        
        if (content != null)
        {
            String dialect = content.getAttribute(WsnConstants.DIALECT);
            String pattern = XmlUtils.extractText(content);
            return new MessagePatternFilter(pattern, dialect);
        }
        
        //
        // no match for known filter types - try and tell the user what 
        // was found and why it's not valid
        //
        Object[] filler = { XmlUtils.getFirstElement(xml) };
        throw new InvalidFilterFault(_MESSAGES.get("InvalidFilterType", filler));
    }
}
