/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.notification.topics.impl;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;

import javax.xml.namespace.QName;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.notification.faults.InvalidTopicExpressionFault;
import org.apache.muse.ws.notification.topics.Topic;
import org.apache.muse.ws.notification.topics.TopicPathExpression;
import org.apache.muse.ws.notification.topics.TopicNamespace;
import org.apache.muse.ws.notification.topics.TopicSet;

/**
 *
 * ConcretePathExpression allows users to evaluate simple and concrete 
 * topic expressions, as defined by WS-Topics v1.3. This class 
 * cannot evaluate paths with wildcards ('full' expressions).
 *
 * @author Dan Jemiolo (danj)
 *
 */

public class ConcretePathExpression implements TopicPathExpression
{    
    private static Messages _MESSAGES = MessagesFactory.get(ConcretePathExpression.class);
        
    private QName _topicPath = null;
    
    private TopicNamespace _topicSpace = null;
    
    public ConcretePathExpression(Topic topic)
    {
        if (topic == null)
            throw new NullPointerException(_MESSAGES.get("NullTopic"));
        
        _topicSpace = topic.getTopicNamespace();
        _topicPath = createTopicPath(_topicSpace, topic);
    }
    
    public ConcretePathExpression(TopicNamespace topicSpace, QName topicPath)
        throws InvalidTopicExpressionFault
    {
        if (topicSpace == null)
            throw new NullPointerException(_MESSAGES.get("NullTopicSpace"));

        if (topicPath == null)
            throw new NullPointerException(_MESSAGES.get("NullTopicPath"));
        
        _topicSpace = topicSpace;
        _topicPath = topicPath;

        String topicNS = topicPath.getNamespaceURI();
        String targetNS = _topicSpace.getTargetNamespace();
        
        if (!targetNS.equals(topicNS))
        {
            Object[] filler = { topicNS, targetNS };
            throw new InvalidTopicExpressionFault(_MESSAGES.get("InvalidTopicNS", filler));
        }
        
        validateTopicPath(_topicPath);
    }
    
    public ConcretePathExpression(TopicSet topics, QName topicPath)
        throws InvalidTopicExpressionFault
    {
        if (topics == null)
            throw new NullPointerException(_MESSAGES.get("NullTopicSpaceForest"));
        
        if (topicPath == null)
            throw new NullPointerException(_MESSAGES.get("NullTopicPath"));
        
        _topicPath = topicPath;
        
        String uri = _topicPath.getNamespaceURI();
        _topicSpace = topics.getTopicNamespace(uri);
        
        if (_topicSpace == null)
        {
            Object[] filler = { uri };
            throw new InvalidTopicExpressionFault(_MESSAGES.get("InvalidTopicSpaceNS", filler));
        }

        String topicNS = topicPath.getNamespaceURI();
        String targetNS = _topicSpace.getTargetNamespace();
        
        if (!targetNS.equals(topicNS))
        {
            Object[] filler = { topicNS, targetNS };
            throw new InvalidTopicExpressionFault(_MESSAGES.get("InvalidTopicNS", filler));
        }
        
        validateTopicPath(_topicPath);
    }
    
    private QName createTopicPath(TopicNamespace topicSpace, Topic topic)
    {
        String namespace = topicSpace.getTargetNamespace();
        
        LinkedList nodes = new LinkedList();
        
        String name = topic.getName();
        int totalSize = name.length();
        nodes.add(name);
        
        Topic parent = topic.getParentTopic();
        
        while (parent != null)
        {
            name = parent.getName();
            totalSize += name.length();
            
            nodes.addFirst(name);
            parent = parent.getParentTopic();
        }
        
        StringBuffer buffer = new StringBuffer(totalSize);
        Iterator i = nodes.iterator();
        
        while (i.hasNext())
        {
            buffer.append(i.next());
            
            if (i.hasNext())
                buffer.append('/');
        }
        
        return new QName(namespace, buffer.toString(), "topic");
    }
        
    public QName getTopicPath()
    {
        return _topicPath;
    }
    
    public TopicNamespace getTopicSpace()
    {
        return _topicSpace;
    }
    
    public ListIterator iterator()
    {
        String[] nodes = _topicPath.getLocalPart().split("/");
        return new ConcretePathIterator(_topicSpace, nodes);
    }

    public Topic resolve()
    {
        //
        // all we have to do is use an iterator to move through the path; if 
        // we can get through each node with the iterator without hitting a 
        // dead end, then we have found a match
        //
        
        ListIterator i = iterator();
        Topic topic = null;
        
        while (i.hasNext())
        {
            topic = (Topic)i.next();
            
            //
            // path could not be completed - no match
            //
            if (topic == null)
                return null;
        }
        
        return topic;
    }
    
    /**
     * 
     * Validates the given topic name against the rules for concrete topic 
     * expressions.
     * 
     * @throws InvalidTopicExpressionFault
     *         <ul>
     *         <li>If the expression violates the rules for concrete topic 
     *         expressions as outlined in WS-Topics v1.3.
     *         </ul>
     *
     */
    public static void validateTopicPath(QName topicPath) 
        throws InvalidTopicExpressionFault
    {        
        String unqualifiedPath = topicPath.getLocalPart();
        
        if (unqualifiedPath.indexOf('*') >= 0 || 
            unqualifiedPath.indexOf("//") >= 0)
        {
            Object[] filler = { XmlUtils.toString(topicPath) };
            throw new InvalidTopicExpressionFault(_MESSAGES.get("InvalidConcretePath", filler));
        }
    }
    
    /**
     * 
     * ConcretePathIterator is an internal class that allows us to move back 
     * and forth between nodes in a topic path. Each node is one of the strings 
     * between the '/' characters in the path.
     * 
     * @author Dan Jemiolo (danj)
     * 
     */
   class ConcretePathIterator implements ListIterator
   {
       //
       // position of the iterator in the path
       //
       private int _currentNode = 0;
       
       //
       // the topic representing the iterator's current place on the path
       //
       private Topic _currentTopic = null;
       
       //
       // the set of items the iterator can move over
       //
       private String[] _nodes = null;
       
       private TopicNamespace _root = null;
       
       public ConcretePathIterator(TopicNamespace root, String[] nodes)
       {
           _root = root;
           _nodes = nodes;
       }
       
       public void add(Object item)
       {
           throw new UnsupportedOperationException();
       }

       public boolean hasNext()
       {
           return _currentNode < _nodes.length;
       }
       
       public boolean hasPrevious()
       {
           return _currentNode > 0;
       }

       public Object next()
       {
           if (!hasNext())
               throw new IllegalStateException();
           
           String name = _nodes[_currentNode++];
           
           //
           // special case - if we're at the beginning, we have 
           // to get the first topic from the TopicNamespace
           //
           if (_currentTopic == null)
               _currentTopic = _root.getTopic(name);
           
           else
               _currentTopic = _currentTopic.getTopic(name);
           
           return _currentTopic;
       }
       
       public int nextIndex()
       {
           return _currentNode;
       }
       
       public Object previous()
       {
           if (!hasPrevious())
               throw new IllegalStateException();
           
           --_currentNode;
           return _currentTopic.getParentTopic();
       }
       
       public int previousIndex()
       {
           return _currentNode - 1;
       }

       public void remove()
       {
           throw new UnsupportedOperationException();
       }

       public void set(Object item)
       {
           throw new UnsupportedOperationException();
       }
   }
}
