/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.resource.metadata;

import java.util.Collection;

import javax.xml.namespace.QName;

import org.apache.muse.util.xml.XmlSerializable;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 *
 * MetadataDescriptor is an interface for a single metadata descriptor. 
 * It defines a lookup service for property metadata; in addition to 
 * actual metadata values, the interface has convenience methods for 
 * answering common questions ("Can I delete this property?", etc.) 
 * rather than requiring the user to calculate the answers based on 
 * metadata values. This interface is based on WS-RMD 1.0.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public interface MetadataDescriptor extends XmlSerializable
{
    /**
     * 
     * @param property
     * 
     * @return True if the property can be deleted (it is read-write and 
     *         mutable)
     *
     */
    boolean canDelete(QName property);

    /**
     * 
     * @param property
     * 
     * @return True if the property can have new instances inserted (it is 
     *         read-write and mutable/appendable).
     *
     */
    boolean canInsert(QName property);
    
    /**
     * 
     * @param property
     * 
     * @return True if the property's instances can be updated (it is 
     *         read-write and mutable).
     *
     */
    boolean canUpdate(QName property);
    
    /**
     * 
     * This method allows you to extend a wsrmd:Property definition and provide 
     * application-specific metadata for a property. These will be parsed as 
     * simple name-value pairs - you cannot associate complex types or XML 
     * fragments with an extended metadata item.
     *
     * @param property
     * 
     * @param elementName
     *        The name of the extended metadata item.
     * 
     * @return The value of the extended metadata item, or null if there is no 
     *         such item.
     *
     */
    String getExtendedMetadata(QName property, QName elementName);
    
    /**
     * 
     * @param property
     * 
     * @return The collection of initial values defined for this property.
     *         <br><br>
     *         The actual values are DOM Elements, since that is how they are 
     *         described in the RMD. They can be compared to the results of a 
     *         WS-RP GetResourceProperty call.
     * 
     * @see #getInitialValues(QName, Class)
     * 
     */
    Collection getInitialValues(QName property);
    
    /**
     * 
     * @param property
     * 
     * @param javaType
     *        The type whose serializer should be used to convert the 
     *        property values (DOM Elements) into POJOs.
     * 
     * @return The array of getInitialValues(QName) in POJO form.
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If there is no Serializer registered for the given type.</li>
     *         <li>If the property value XML can not be translated into an 
     *         instance of the given type.</li>
     *         </ul>
     * 
     * @see #getValidValues(QName)
     *
     */
    Collection getInitialValues(QName property, Class javaType)
        throws SoapFault;
    
    QName getInterface();
    
    /**
     * 
     * @param property
     * 
     * @return The lower bound of the property's valid value range, if one 
     *         is defined. The user must interpret the string value in the 
     *         context of the property's type. The method returns null if 
     *         no range is defined.
     *
     */
    String getLowerBound(QName property);
    
    /**
     * 
     * @return The name of the metadata descriptor (defined in XML by its 
     *         "name" attribute).
     *
     */
    String getName();
    
    /**
     *
     * @return The collection of property names that are defined in this 
     *         descriptor.
     *
     */
    Collection getPropertyNames();
    
    /**
     * 
     * @param property
     * 
     * @return The collection of static values defined for this property. If 
     *         no static values exist, the collection will be empty.
     *         <br><br>
     *         The actual values are DOM Elements, since that is how they are 
     *         described in the RMD. They can be compared to the results of a 
     *         WS-RP GetResourceProperty call.
     * 
     * @see #getStaticValues(QName, Class)
     *
     */
    Collection getStaticValues(QName property);
    
    /**
     * 
     * @param property
     * 
     * @param javaType
     *        The type whose serializer should be used to convert the 
     *        property values (DOM Elements) into POJOs.
     * 
     * @return The array of getStaticValues(QName) in POJO form.
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If there is no Serializer registered for the given type.</li>
     *         <li>If the property value XML can not be translated into an 
     *         instance of the given type.</li>
     *         </ul>
     * 
     * @see #getStaticValues(QName)
     *
     */
    Collection getStaticValues(QName property, Class javaType)
        throws SoapFault;
    
    /**
     * 
     * @param property
     * 
     * @return The upper bound of the property's valid value range, if one 
     *         is defined. The user must interpret the string value in the 
     *         context of the property's type. The method returns null if 
     *         no range is defined.
     *
     */
    String getUpperBound(QName property);
    
    /**
     * 
     * @param property
     * 
     * @return The collection of valid values defined for this property. If 
     *         no valid values exist, the collection will be empty (in this 
     *         case, <b>all</b> values are valid).
     *         <br><br>
     *         The actual values are DOM Elements, since that is how they are 
     *         described in the RMD (unlike a valid value range, the valid 
     *         values are complete property instances). They can be compared 
     *         to the results of a WS-RP GetResourceProperty call.
     * 
     * @see #getValidValues(QName, Class)
     * 
     */
    Collection getValidValues(QName property);
    
    /**
     * 
     * @param property
     * 
     * @param javaType
     *        The type whose serializer should be used to convert the 
     *        property values (DOM Elements) into POJOs.
     * 
     * @return The array of getValidValues(QName) in POJO form.
     * 
     * @throws SoapFault
     *         <ul>
     *         <li>If there is no Serializer registered for the given type.</li>
     *         <li>If the property value XML can not be translated into an 
     *         instance of the given type.</li>
     *         </ul>
     * 
     * @see #getValidValues(QName)
     *
     */
    Collection getValidValues(QName property, Class javaType)
        throws SoapFault;
    
    String getWsdlLocation();
    
    /**
     * 
     * @param property
     * 
     * @return True if the descriptor defines a property with the given name. 
     *
     */
    boolean hasProperty(QName property);
    
    /**
     * 
     * @param property
     * @param value
     * 
     * @return True if the given value is found in the property's set of 
     *         initial values. If the property has no initial values defined, 
     *         the method returns "false".
     *
     */
    boolean isInitialValue(QName property, Object value);
    
    /**
     * 
     * @param property
     * 
     * @return True if the property cannot be modified by external clients, 
     *         but is mutable by internal code (it is read-only but mutable).
     *
     */
    boolean isReadOnlyExternal(QName property);
    
    /**
     * 
     * @param property
     * @param value
     * 
     * @return True if the given value is found in the property's set of 
     *         static values. If the property has no static values defined, 
     *         the method returns "false".
     *
     */
    boolean isStaticValue(QName property, Object value);
    
    /**
     * 
     * @param property
     * @param value
     * 
     * @return True if the given value is found in the property's set of 
     *         valid values. If the property has no valid values defined, 
     *         the method returns "true".
     *
     */
    boolean isValidValue(QName property, Object value);
}
