/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.resource.metadata;

import java.util.Collection;
import java.util.Collections;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.muse.util.xml.XmlUtils;

/**
 *
 * OpenMetadataDescriptor is a {@linkplain MetadataDescriptor MetadataDescriptor} 
 * that allows free reign on all properties, since no rules or restrictions have 
 * been put in place. This class is a singleton because an empty RMD is the same 
 * no matter what properties are defined.
 * 
 * @author Dan Jemiolo (danj)
 *
 */

public class OpenMetadataDescriptor implements MetadataDescriptor
{
    //
    // This descriptor is the same no matter what document it's applied to, 
    // so we only need one instance of it
    //
    private static final OpenMetadataDescriptor _SINGLETON = new OpenMetadataDescriptor();
    
    /**
     * 
     * @return The singleton instance of this class.
     *
     */
    public static OpenMetadataDescriptor getInstance()
    {
        return _SINGLETON;
    }
    
    /**
     * 
     * Private constructor used to prevent instantiation.
     *
     */
    private OpenMetadataDescriptor()
    {
        //
        // this constructor is only explicit so we can make it private
        //
    }
    
    /**
     *
     * @return true
     * 
     */
    public boolean canDelete(QName property)
    {
        return true;
    }

    /**
     *
     * @return true
     * 
     */
    public boolean canInsert(QName property)
    {
        return true;
    }

    /**
     *
     * @return true
     * 
     */
    public boolean canUpdate(QName property)
    {
        return true;
    }

    public String getExtendedMetadata(QName property, QName elementName)
    {
        return null;
    }

    /**
     *
     * @return An empty collection.
     * 
     */
    public Collection getInitialValues(QName property)
    {
        return Collections.EMPTY_LIST;
    }

    /**
     *
     * @return An empty collection.
     * 
     */
    public Collection getInitialValues(QName property, Class type)
    {
        return Collections.EMPTY_LIST;
    }

    public QName getInterface()
    {
        return null;
    }

    /**
     *
     * @return null
     * 
     */
    public String getLowerBound(QName property)
    {
        return null;
    }

    /**
     *
     * @return null
     * 
     */
    public String getName()
    {
        return null;
    }

    /**
     *
     * @return An empty collection.
     * 
     */
    public Collection getPropertyNames()
    {
        return Collections.EMPTY_LIST;
    }

    /**
     *
     * @return An empty collection.
     * 
     */
    public Collection getStaticValues(QName property)
    {
        return Collections.EMPTY_LIST;
    }

    /**
     *
     * @return An empty collection.
     * 
     */
    public Collection getStaticValues(QName property, Class type)
    {
        return Collections.EMPTY_LIST;
    }

    /**
     *
     * @return null
     * 
     */
    public String getUpperBound(QName property)
    {
        return null;
    }

    /**
     *
     * @return An empty collection.
     * 
     */
    public Collection getValidValues(QName property)
    {
        return Collections.EMPTY_LIST;
    }

    /**
     *
     * @return An empty collection.
     * 
     */
    public Collection getValidValues(QName property, Class type)
    {
        return Collections.EMPTY_LIST;
    }

    public String getWsdlLocation()
    {
        return null;
    }

    /**
     *
     * @return true
     * 
     */
    public boolean hasProperty(QName property)
    {
        return true;
    }

    /**
     *
     * @return false
     * 
     */
    public boolean isInitialValue(QName property, Object value)
    {
        return false;
    }

    /**
     *
     * @return false
     * 
     */
    public boolean isReadOnlyExternal(QName property)
    {
        return false;
    }

    /**
     *
     * @return false
     * 
     */
    public boolean isStaticValue(QName property, Object value)
    {
        return false;
    }

    /**
     *
     * @return true
     * 
     */
    public boolean isValidValue(QName property, Object value)
    {
        return true;
    }

    public Element toXML()
    {
        return toXML(XmlUtils.EMPTY_DOC);
    }

    public Element toXML(Document doc)
    {
        return XmlUtils.createElement(doc, WsrmdConstants.DESCRIPTOR_QNAME);
    }
}
