/*=============================================================================*
 *  Copyright 2006 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/

package org.apache.muse.ws.resource.properties.schema;

import java.util.Collection;
import java.util.Collections;

import javax.xml.namespace.QName;

import org.apache.muse.ws.resource.WsResourceCapability;
import org.apache.muse.ws.resource.properties.WsrpConstants;

/**
 *
 * OpenPropertiesSchema is a {@linkplain ResourcePropertiesSchema ResourcePropertiesSchema} 
 * that has no restrictions on any of its properties - all operations are 
 * allowed, and no values have limits. This class is a singleton because a 
 * restriction-less schema is the same no matter what properties are defined.
 *
 * @author Dan Jemiolo (danj)
 *
 */

public final class OpenPropertiesSchema implements ResourcePropertiesSchema
{
    //
    // This schema is the same no matter what document it's applied to, 
    // so we only need one instance of it
    //
    private static final OpenPropertiesSchema _SINGLETON = new OpenPropertiesSchema();
    
    /**
     * 
     * @return The singleton instance of this class.
     *
     */
    public static OpenPropertiesSchema getInstance()
    {
        return _SINGLETON;
    }
    
    /**
     * 
     * Private constructor used to prevent instantiation.
     *
     */
    private OpenPropertiesSchema()
    {
        //
        // this constructor is only explicit so we can make it private
        //
    }
    
    public WsResourceCapability getCapability(QName property)
    {
        throw new UnsupportedOperationException();
    }

    public QName getElementName()
    {
        return WsrpConstants.DEFAULT_DOCUMENT_QNAME;
    }
    
    /**
     * 
     * @return ResourcePropertiesSchema.UNBOUNDED
     *
     */
    public int getMaxOccurs(QName property)
    {
        return UNBOUNDED;
    }
    
    /**
     * 
     * @return zero (0)
     *
     */
    public int getMinOccurs(QName property)
    {
        return 0;
    }
    
    /**
     * 
     * @return An empty collection.
     *
     */
    public Collection getPropertyNames()
    {
        return Collections.EMPTY_LIST;
    }

    /**
     * 
     * @return null
     *
     */
    public QName getPropertyTypeName(QName property)
    {
        return null;
    }
    
    public boolean hasCapability(QName property)
    {
        return true;
    }

    /**
     * 
     * @return true
     *
     */
    public boolean hasProperty(QName property)
    {
        return true;
    }

    /**
     * 
     * @return true
     *
     */
    public boolean isMaxUnbounded(QName property)
    {
        return true;
    }

    /**
     * 
     * @return true
     *
     */
    public boolean isNillable(QName property)
    {
        return true;
    }

    public void setCapability(QName property, WsResourceCapability capability)
    {
        throw new UnsupportedOperationException();
    }

    public void setElementName(QName wsrpName)
    {
        throw new UnsupportedOperationException();
    }
}
